Package: simtrial
Type: Package
Title: Clinical Trial Simulation
Version: 1.0.2
Authors@R: c(
    person("Keaven", "Anderson", email = "keaven_anderson@merck.com", role = c("aut")),
    person("Yujie", "Zhao", email = "yujie.zhao@merck.com", role =  c("aut","cre")),
    person("John", "Blischak", role = c("aut")),
    person("Nan", "Xiao", role = c("ctb")),
    person("Yilong", "Zhang", role =  c("aut")),
    person("Jianxiao", "Yang", role = c("ctb")),
    person("Lili", "Ling", role = c("ctb")),
    person("Xintong", "Li", role = c("ctb")),
    person("Ruixue", "Wang", role = c("ctb")),
    person("Yi", "Cui", role = c("ctb")),
    person("Ping", "Yang", role = c("ctb")),
    person("Yalin", "Zhu", role = c("ctb")),
    person("Heng", "Zhou", role = c("ctb")),
    person("Amin", "Shirazi", role = c("ctb")),
    person("Cole", "Manschot", role = c("ctb")),
    person("Larry", "Leon", role = c("ctb")),
    person("Merck & Co., Inc., Rahway, NJ, USA and its affiliates", role = "cph",
           comment = c(ROR = "02891sr49"))
    )
Description: Provides some basic routines for simulating a
    clinical trial. The primary intent is to provide some tools to
    generate trial simulations for trials with time to event outcomes.
    Piecewise exponential failure rates and piecewise constant
    enrollment rates are the underlying mechanism used to simulate
    a broad range of scenarios such as those presented in
    Lin et al. (2020) <doi:10.1080/19466315.2019.1697738>.
    However, the basic generation of data is done using pipes to allow
    maximum flexibility for users to meet different needs.
License: GPL-3
URL: https://merck.github.io/simtrial/,
        https://github.com/Merck/simtrial
BugReports: https://github.com/Merck/simtrial/issues
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: Rcpp, data.table (>= 1.12.4), doFuture, foreach, future,
        methods, mvtnorm, stats, survival, utils
Suggests: Matrix, covr, dplyr, ggplot2, gsDesign, gsDesign2 (>= 1.1.4),
        gt, knitr, rmarkdown, survMisc, survRM2, testthat (>= 3.0.0),
        tibble, tidyr
LinkingTo: Rcpp
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-11-20 21:29:51 UTC; zhayuji4
Author: Keaven Anderson [aut],
  Yujie Zhao [aut, cre],
  John Blischak [aut],
  Nan Xiao [ctb],
  Yilong Zhang [aut],
  Jianxiao Yang [ctb],
  Lili Ling [ctb],
  Xintong Li [ctb],
  Ruixue Wang [ctb],
  Yi Cui [ctb],
  Ping Yang [ctb],
  Yalin Zhu [ctb],
  Heng Zhou [ctb],
  Amin Shirazi [ctb],
  Cole Manschot [ctb],
  Larry Leon [ctb],
  Merck & Co., Inc., Rahway, NJ, USA and its affiliates [cph] (ROR:
    <https://ror.org/02891sr49>)
Maintainer: Yujie Zhao <yujie.zhao@merck.com>
Repository: CRAN
Date/Publication: 2025-11-20 22:10:03 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 07:58:46 UTC; unix
Archs: simtrial.so.dSYM
