# Generated by roxygen2: do not edit by hand

S3method(as_gt,fixed_design)
S3method(as_gt,gs_design)
S3method(as_gt,simtrial_gs_wlr)
S3method(summary,simtrial_gs_wlr)
S3method(wlr,counting_process)
S3method(wlr,default)
S3method(wlr,tte_data)
export(as_gt)
export(counting_process)
export(create_cut)
export(create_test)
export(cut_data_by_date)
export(cut_data_by_event)
export(early_zero)
export(fh)
export(fit_pwexp)
export(get_analysis_date)
export(get_cut_date_by_event)
export(maxcombo)
export(mb)
export(milestone)
export(multitest)
export(randomize_by_fixed_block)
export(rmst)
export(rpwexp)
export(rpwexp_enroll)
export(sim_fixed_n)
export(sim_gs_n)
export(sim_pw_surv)
export(to_sim_pw_surv)
export(wlr)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,dcast)
importFrom(data.table,fifelse)
importFrom(data.table,frankv)
importFrom(data.table,last)
importFrom(data.table,merge.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(data.table,uniqueN)
importFrom(doFuture,"%dofuture%")
importFrom(future,nbrOfWorkers)
importFrom(future,plan)
importFrom(methods,is)
importFrom(mvtnorm,GenzBretz)
importFrom(mvtnorm,pmvnorm)
importFrom(survival,Surv)
importFrom(survival,is.Surv)
importFrom(survival,strata)
importFrom(survival,survfit)
useDynLib(simtrial, .registration = TRUE)
